@@ -20,10 +20,12 @@ if test ! -x "`which aclocal`"
 then echo you need autoconfig to generate the Makefile
 fi
 
+sys=`uname -s`
+
 libtoolize --force --copy
 # autoheader
+acinclude
 aclocal
-# automake --add-missing --copy --gnu
 autoconf
-./configure $*
 
+./configure ${CFGARGS}  $*
