@@ -6,7 +6,7 @@
 
   Author:  Vadim Zaliva lord@crocodile.org
 
-  $Id: snmp.cpp,v 1.2 2004/09/08 05:34:53 vkurland Exp $
+  $Id: snmp.cpp,v 1.6 2005/05/03 05:23:37 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -56,6 +56,7 @@
 #include <typeinfo>
 #include <algorithm>
 #include <memory>
+#include <sstream>
 
 /**
  * Define this if you need extra debug output.
@@ -65,6 +66,11 @@
 using namespace std;
 using namespace libfwbuilder;
 
+/*
+ *  symbolic OID names - this requires MIB installed on the system,
+ *  which may be a problem. In particular, we do not ship MIBs with
+ *  windows package so the symbolic OID names won't work there at all
+
 const char *SNMPQuery::SNMP_INTERFACE_ASTATUS= "interfaces.ifTable.ifEntry.ifAdminStatus"  ;
 const char *SNMPQuery::SNMP_INTERFACE_OSTATUS= "interfaces.ifTable.ifEntry.ifOperStatus"   ;
 const char *SNMPQuery::SNMP_INTERFACE_INDEX  = "interfaces.ifTable.ifEntry.ifIndex"        ;
@@ -95,6 +101,36 @@ const char *SNMPQuery::SNMP_ROUTE_TYPE_TABLE = "ip.ipRouteTable.ipRouteEntry.ipR
 const char *SNMPQuery::SNMP_ROUTE_IF_TABLE   = "ip.ipRouteTable.ipRouteEntry.ipRouteIfIndex" ;
 const char *SNMPQuery::SNMP_ROUTE_GW_TABLE   = "ip.ipRouteTable.ipRouteEntry.ipRouteNextHop" ;
 const int   SNMPQuery::SNMP_DIRECT_ROUTE     = 3;
+*/
+
+
+/* Compiled OIDs */
+const char *SNMPQuery::SNMP_INTERFACE_ASTATUS= ".1.3.6.1.2.1.2.2.1.7"  ;
+const char *SNMPQuery::SNMP_INTERFACE_OSTATUS= ".1.3.6.1.2.1.2.2.1.8"   ;
+const char *SNMPQuery::SNMP_INTERFACE_INDEX  = ".1.3.6.1.2.1.2.2.1.1"        ;
+const char *SNMPQuery::SNMP_INTERFACES_DESCR = ".1.3.6.1.2.1.2.2.1.2"        ;
+const char *SNMPQuery::SNMP_INTERFACES_PHYSA = ".1.3.6.1.2.1.2.2.1.6"  ;
+const char *SNMPQuery::SNMP_INTERFACES_TYPE  = ".1.3.6.1.2.1.2.2.1.3"         ;
+const char *SNMPQuery::SNMP_ADDR_INDEX_TABLE = ".1.3.6.1.2.1.4.20.1.2" ;
+const char *SNMPQuery::SNMP_NMASK_TABLE      = ".1.3.6.1.2.1.4.20.1.3" ;
+const char *SNMPQuery::SNMP_ADDR_TABLE       = ".1.3.6.1.2.1.4.20.1.1"    ;
+const char *SNMPQuery::SNMP_BCAST_TABLE      = ".1.3.6.1.2.1.4.20.1.4" ;
+const char *SNMPQuery::SNMP_AT_TABLE_NET     = ".1.3.6.1.2.1.3.1.1.3";
+const char *SNMPQuery::SNMP_AT_TABLE_PHYS    = ".1.3.6.1.2.1.3.1.1.2";
+
+const char *SNMPQuery::SNMP_SYSNAME          = ".1.3.6.1.2.1.1.5.0"     ;
+const char *SNMPQuery::SNMP_SYSDESCR         = ".1.3.6.1.2.1.1.1.0"    ;
+const char *SNMPQuery::SNMP_CONTACT          = ".1.3.6.1.2.1.1.4.0"  ;
+const char *SNMPQuery::SNMP_LOCATION         = ".1.3.6.1.2.1.1.6.0" ;
+
+const char *SNMPQuery::SNMP_ROUTE_DST_TABLE  = ".1.3.6.1.2.1.4.21.1.1"    ;
+const char *SNMPQuery::SNMP_ROUTE_NM_TABLE   = ".1.3.6.1.2.1.4.21.1.11"    ;
+const char *SNMPQuery::SNMP_ROUTE_TYPE_TABLE = ".1.3.6.1.2.1.4.21.1.8"    ;
+const char *SNMPQuery::SNMP_ROUTE_IF_TABLE   = ".1.3.6.1.2.1.4.21.1.2" ;
+const char *SNMPQuery::SNMP_ROUTE_GW_TABLE   = ".1.3.6.1.2.1.4.21.1.7" ;
+const int   SNMPQuery::SNMP_DIRECT_ROUTE     = 3;
+
+
 
 /**
  * Following interface types correspond to point-to-point interfaces.
@@ -139,6 +175,9 @@ void SNMPQuery::init(string h, string c, int retries_, long timeout_)
     descr     = "" ;
     contact   = "" ;
     location  = "" ;
+
+    char *snmp_out_opt = "n";
+    snmp_out_toggle_options(snmp_out_opt);
 }
 
 SNMPQuery::~SNMPQuery()
@@ -329,11 +368,7 @@ void SNMPQuery::fetchRoutingTable(Logger *logger,SyncFlag *stop_program, SNMPCon
                     continue;
                 }
                 IPAddress dst = dynamic_cast<SNMPVariable_IPaddr*>((*j).second)->getIPAddressValue();
-                string rname;
-		if ((*j).first.substr(0,strlen(RFC1213_MIB_PREFIX_IPROUTEDEST_PREFIX))==RFC1213_MIB_PREFIX_IPROUTEDEST_PREFIX)
-		  rname=(*j).first.substr(strlen(RFC1213_MIB_PREFIX_IPROUTEDEST_PREFIX)+1);
-		else
-		  rname=(*j).first.substr(strlen(SNMP_ROUTE_DST_TABLE)+1);
+                string rname=(*j).first.substr(strlen(SNMP_ROUTE_DST_TABLE)+1);
 
                 CHECK_STOP
                 v=c->get(string(SNMP_ROUTE_NM_TABLE)+"."+rname);
@@ -487,11 +522,8 @@ void SNMPQuery::fetchInterfaces(Logger *logger,SyncFlag *stop_program, SNMPConne
                 continue;
             }
             long ind = SNMPVariable::var2Int((*j).second);
-	    string iname;
-	    if ((*j).first.substr(0,strlen(IP_MIB_ENTIFINDEX_PREFIX))==IP_MIB_ENTIFINDEX_PREFIX)
-	      iname=(*j).first.substr(strlen(IP_MIB_ENTIFINDEX_PREFIX)+1);
-	    else
-              iname=(*j).first.substr(strlen(SNMP_ADDR_INDEX_TABLE)+1);
+
+	    string iname=(*j).first.substr(strlen(SNMP_ADDR_INDEX_TABLE)+1);
 
 /*
  * Example (two interfaces, first has two addresses):
@@ -854,7 +886,9 @@ multimap<string, SNMPVariable* > SNMPConnection::walk(const string &variable) th
     size_t rootlen;
     
     rootlen = MAX_OID_LEN;
-    get_node(variable.c_str(), root, &rootlen); //TODO: error check
+//    get_node(variable.c_str(), root, &rootlen); //TODO: error check
+
+    read_objid(variable.c_str(), root, &rootlen); //TODO: error check
 
     oid    name[MAX_OID_LEN];
     size_t name_length;
@@ -871,7 +905,7 @@ multimap<string, SNMPVariable* > SNMPConnection::walk(const string &variable) th
         snmp_add_null_var(pdu, name, name_length);
         
         /* do the request */
-        struct snmp_pdu *response;
+        struct snmp_pdu *response = NULL;
         int status = snmp_synch_response(session, pdu, &response);
         if(status == STAT_SUCCESS)
         {
@@ -917,19 +951,32 @@ multimap<string, SNMPVariable* > SNMPConnection::walk(const string &variable) th
                     throw FWException("SNMPSession: error: End of MIB");
                 } else 
                 {
-                    throw FWException(string("SNMPSession: Error in packet. Reason:")+
+                    throw FWException(string("SNMPSession: SNMP error: ")+
                                       snmp_errstring(response->errstat));
                 }
             }
-        } else if (status == STAT_TIMEOUT)
-        {
-            throw FWException("SNMPSession: SNMP timeout");
-        } else 
-        {    /* status == STAT_ERROR */
-            throw FWException("SNMPSession: SNMP error");
+//        } else if (status == STAT_TIMEOUT)
+//        {
+//            throw FWException("SNMPSession: SNMP timeout");
+        } else
+        {    // status == STAT_ERROR
+            if (response != NULL)
+            {
+                throw FWException(string("SNMPSession: SNMP error: '") +
+                                  string(snmp_errstring(response->errstat)) + "'");
+                snmp_free_pdu(response);
+            } else 
+            {
+                std::ostringstream str;
+                str << "SNMPSession: SNMP error, status ";
+                str << status;
+                int liberr,syserr;
+                char *errstr = "";
+                snmp_error(session, &liberr, &syserr, &errstr);
+                str << " " << errstr;
+                throw FWException(str.str());
+            }
         }
-        if (response)
-            snmp_free_pdu(response);
     }
     
     return res;
@@ -943,7 +990,8 @@ vector<SNMPVariable*> SNMPConnection::get(const string &variable) throw(FWExcept
     struct snmp_pdu *pdu = snmp_pdu_create(SNMP_MSG_GET);
     oid anOID[MAX_OID_LEN];
     size_t anOID_len = MAX_OID_LEN;
-    get_node(variable.c_str(), anOID, &anOID_len); //TODO: error check
+//    get_node(variable.c_str(), anOID, &anOID_len); //TODO: error check
+    read_objid(variable.c_str(), anOID, &anOID_len); //TODO: error check
     snmp_add_null_var(pdu, anOID, anOID_len);
     struct snmp_pdu *response;
     int status = snmp_synch_response(session, pdu, &response);
